# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("wsnmp32.dll")
prototypes = \
    {
        # 
        'SnmpGetTranslateMode': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="SNMP_API_TRANSLATE_MODE"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["nTranslateMode"]),
        # 
        'SnmpSetTranslateMode': SimTypeFunction([SimTypeInt(signed=False, label="SNMP_API_TRANSLATE_MODE")], SimTypeInt(signed=False, label="UInt32"), arg_names=["nTranslateMode"]),
        # 
        'SnmpGetRetransmitMode': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="SNMP_STATUS"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["nRetransmitMode"]),
        # 
        'SnmpSetRetransmitMode': SimTypeFunction([SimTypeInt(signed=False, label="SNMP_STATUS")], SimTypeInt(signed=False, label="UInt32"), arg_names=["nRetransmitMode"]),
        # 
        'SnmpGetTimeout': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hEntity", "nPolicyTimeout", "nActualTimeout"]),
        # 
        'SnmpSetTimeout': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hEntity", "nPolicyTimeout"]),
        # 
        'SnmpGetRetry': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hEntity", "nPolicyRetry", "nActualRetry"]),
        # 
        'SnmpSetRetry': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hEntity", "nPolicyRetry"]),
        # 
        'SnmpGetVendorInfo': SimTypeFunction([SimTypePointer(SimStruct({"vendorName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 64), "vendorContact": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 64), "vendorVersionId": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 32), "vendorVersionDate": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 32), "vendorEnterprise": SimTypeInt(signed=False, label="UInt32")}, name="smiVENDORINFO", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["vendorInfo"]),
        # 
        'SnmpStartup': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="SNMP_API_TRANSLATE_MODE"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="SNMP_STATUS"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["nMajorVersion", "nMinorVersion", "nLevel", "nTranslateMode", "nRetransmitMode"]),
        # 
        'SnmpCleanup': SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")),
        # 
        'SnmpOpen': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hWnd", "wMsg"]),
        # 
        'SnmpClose': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["session"]),
        # 
        'SnmpSendMsg': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["session", "srcEntity", "dstEntity", "context", "PDU"]),
        # 
        'SnmpRecvMsg': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["session", "srcEntity", "dstEntity", "context", "PDU"]),
        # 
        'SnmpRegister': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="smiOID", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="SNMP_STATUS")], SimTypeInt(signed=False, label="UInt32"), arg_names=["session", "srcEntity", "dstEntity", "context", "notification", "state"]),
        # 
        'SnmpCreateSession': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hSession", "hWnd", "wMsg", "wParam", "lParam", "lpClientData"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["hWnd", "wMsg", "fCallBack", "lpClientData"]),
        # 
        'SnmpListen': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="SNMP_STATUS")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hEntity", "lStatus"]),
        # 
        'SnmpListenEx': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hEntity", "lStatus", "nUseEntityAddr"]),
        # 
        'SnmpCancelMsg': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["session", "reqId"]),
        # 
        'SnmpStartupEx': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="SNMP_API_TRANSLATE_MODE"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="SNMP_STATUS"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["nMajorVersion", "nMinorVersion", "nLevel", "nTranslateMode", "nRetransmitMode"]),
        # 
        'SnmpCleanupEx': SimTypeFunction([], SimTypeInt(signed=False, label="UInt32")),
        # 
        'SnmpStrToEntity': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["session", "string"]),
        # 
        'SnmpEntityToStr': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["entity", "size", "string"]),
        # 
        'SnmpFreeEntity': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["entity"]),
        # 
        'SnmpStrToContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="smiOCTETS", pack=False, align=None), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["session", "string"]),
        # 
        'SnmpContextToStr': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="smiOCTETS", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["context", "string"]),
        # 
        'SnmpFreeContext': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["context"]),
        # 
        'SnmpSetPort': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hEntity", "nPort"]),
        # 
        'SnmpCreatePdu': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="SNMP_PDU_TYPE"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["session", "PDU_type", "request_id", "error_status", "error_index", "varbindlist"]),
        # 
        'SnmpGetPduData': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="SNMP_PDU_TYPE"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="SNMP_ERROR"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["PDU", "PDU_type", "request_id", "error_status", "error_index", "varbindlist"]),
        # 
        'SnmpSetPduData': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["PDU", "PDU_type", "request_id", "non_repeaters", "max_repetitions", "varbindlist"]),
        # 
        'SnmpDuplicatePdu': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["session", "PDU"]),
        # 
        'SnmpFreePdu': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["PDU"]),
        # 
        'SnmpCreateVbl': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="smiOID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"syntax": SimTypeInt(signed=False, label="UInt32"), "value": SimUnion({"sNumber": SimTypeInt(signed=True, label="Int32"), "uNumber": SimTypeInt(signed=False, label="UInt32"), "hNumber": SimStruct({"hipart": SimTypeInt(signed=False, label="UInt32"), "lopart": SimTypeInt(signed=False, label="UInt32")}, name="smiCNTR64", pack=False, align=None), "string": SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="smiOCTETS", pack=False, align=None), "oid": SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="smiOID", pack=False, align=None), "empty": SimTypeChar(label="Byte")}, name="<anon>", label="None")}, name="smiVALUE", pack=False, align=None), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["session", "name", "value"]),
        # 
        'SnmpDuplicateVbl': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["session", "vbl"]),
        # 
        'SnmpFreeVbl': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["vbl"]),
        # 
        'SnmpCountVbl': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["vbl"]),
        # 
        'SnmpGetVb': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="smiOID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"syntax": SimTypeInt(signed=False, label="UInt32"), "value": SimUnion({"sNumber": SimTypeInt(signed=True, label="Int32"), "uNumber": SimTypeInt(signed=False, label="UInt32"), "hNumber": SimStruct({"hipart": SimTypeInt(signed=False, label="UInt32"), "lopart": SimTypeInt(signed=False, label="UInt32")}, name="smiCNTR64", pack=False, align=None), "string": SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="smiOCTETS", pack=False, align=None), "oid": SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="smiOID", pack=False, align=None), "empty": SimTypeChar(label="Byte")}, name="<anon>", label="None")}, name="smiVALUE", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["vbl", "index", "name", "value"]),
        # 
        'SnmpSetVb': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="smiOID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"syntax": SimTypeInt(signed=False, label="UInt32"), "value": SimUnion({"sNumber": SimTypeInt(signed=True, label="Int32"), "uNumber": SimTypeInt(signed=False, label="UInt32"), "hNumber": SimStruct({"hipart": SimTypeInt(signed=False, label="UInt32"), "lopart": SimTypeInt(signed=False, label="UInt32")}, name="smiCNTR64", pack=False, align=None), "string": SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="smiOCTETS", pack=False, align=None), "oid": SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="smiOID", pack=False, align=None), "empty": SimTypeChar(label="Byte")}, name="<anon>", label="None")}, name="smiVALUE", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["vbl", "index", "name", "value"]),
        # 
        'SnmpDeleteVb': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["vbl", "index"]),
        # 
        'SnmpGetLastError': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["session"]),
        # 
        'SnmpStrToOid': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="smiOID", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["string", "dstOID"]),
        # 
        'SnmpOidToStr': SimTypeFunction([SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="smiOID", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["srcOID", "size", "string"]),
        # 
        'SnmpOidCopy': SimTypeFunction([SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="smiOID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="smiOID", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["srcOID", "dstOID"]),
        # 
        'SnmpOidCompare': SimTypeFunction([SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="smiOID", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="smiOID", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["xOID", "yOID", "maxlen", "result"]),
        # 
        'SnmpEncodeMsg': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="smiOCTETS", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["session", "srcEntity", "dstEntity", "context", "pdu", "msgBufDesc"]),
        # 
        'SnmpDecodeMsg': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="smiOCTETS", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["session", "srcEntity", "dstEntity", "context", "pdu", "msgBufDesc"]),
        # 
        'SnmpFreeDescriptor': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"len": SimTypeInt(signed=False, label="UInt32"), "ptr": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="smiOCTETS", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["syntax", "descriptor"]),
    }

lib.set_prototypes(prototypes)



import logging

from angr.procedures.stubs.format_parser import FormatParser

from cle.backends.externs.simdata.io_file import io_file_data_for_arch

l = logging.getLogger(name=__name__)

######################################
# fprintf
######################################

class fprintf(FormatParser):

    def run(self, file_ptr, fmt):  # pylint:disable=unused-argument
        fd_offset = io_file_data_for_arch(self.state.arch)['fd']
        fileno = self.state.mem[file_ptr + fd_offset:].int.resolved
        simfd = self.state.posix.get_fd(fileno)
        if simfd is None:
            return -1

        # The format str is at index 1
        fmt_str = self._parse(1)
        out_str = fmt_str.replace(2, self.arg)

        simfd.write_data(out_str, out_str.size() // 8)

        return out_str.size() // 8



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("winfax.dll")
prototypes = \
    {
        # 
        'FaxConnectFaxServerA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["MachineName", "FaxHandle"]),
        # 
        'FaxConnectFaxServerW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["MachineName", "FaxHandle"]),
        # 
        'FaxClose': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle"]),
        # 
        'FaxOpenPort': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "DeviceId", "Flags", "FaxPortHandle"]),
        # 
        'FaxCompleteJobParamsA': SimTypeFunction([SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "RecipientNumber": SimTypeBottom(label="PSTR"), "RecipientName": SimTypeBottom(label="PSTR"), "Tsid": SimTypeBottom(label="PSTR"), "SenderName": SimTypeBottom(label="PSTR"), "SenderCompany": SimTypeBottom(label="PSTR"), "SenderDept": SimTypeBottom(label="PSTR"), "BillingCode": SimTypeBottom(label="PSTR"), "ScheduleAction": SimTypeInt(signed=False, label="UInt32"), "ScheduleTime": SimTypeBottom(label="SYSTEMTIME"), "DeliveryReportType": SimTypeInt(signed=False, label="UInt32"), "DeliveryReportAddress": SimTypeBottom(label="PSTR"), "DocumentName": SimTypeBottom(label="PSTR"), "CallHandle": SimTypeInt(signed=False, label="UInt32"), "Reserved": SimTypeFixedSizeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)}, name="FAX_JOB_PARAMA", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "CoverPageName": SimTypeBottom(label="PSTR"), "UseServerCoverPage": SimTypeBottom(label="BOOL"), "RecName": SimTypeBottom(label="PSTR"), "RecFaxNumber": SimTypeBottom(label="PSTR"), "RecCompany": SimTypeBottom(label="PSTR"), "RecStreetAddress": SimTypeBottom(label="PSTR"), "RecCity": SimTypeBottom(label="PSTR"), "RecState": SimTypeBottom(label="PSTR"), "RecZip": SimTypeBottom(label="PSTR"), "RecCountry": SimTypeBottom(label="PSTR"), "RecTitle": SimTypeBottom(label="PSTR"), "RecDepartment": SimTypeBottom(label="PSTR"), "RecOfficeLocation": SimTypeBottom(label="PSTR"), "RecHomePhone": SimTypeBottom(label="PSTR"), "RecOfficePhone": SimTypeBottom(label="PSTR"), "SdrName": SimTypeBottom(label="PSTR"), "SdrFaxNumber": SimTypeBottom(label="PSTR"), "SdrCompany": SimTypeBottom(label="PSTR"), "SdrAddress": SimTypeBottom(label="PSTR"), "SdrTitle": SimTypeBottom(label="PSTR"), "SdrDepartment": SimTypeBottom(label="PSTR"), "SdrOfficeLocation": SimTypeBottom(label="PSTR"), "SdrHomePhone": SimTypeBottom(label="PSTR"), "SdrOfficePhone": SimTypeBottom(label="PSTR"), "Note": SimTypeBottom(label="PSTR"), "Subject": SimTypeBottom(label="PSTR"), "TimeSent": SimTypeBottom(label="SYSTEMTIME"), "PageCount": SimTypeInt(signed=False, label="UInt32")}, name="FAX_COVERPAGE_INFOA", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobParams", "CoverpageInfo"]),
        # 
        'FaxCompleteJobParamsW': SimTypeFunction([SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "RecipientNumber": SimTypeBottom(label="PWSTR"), "RecipientName": SimTypeBottom(label="PWSTR"), "Tsid": SimTypeBottom(label="PWSTR"), "SenderName": SimTypeBottom(label="PWSTR"), "SenderCompany": SimTypeBottom(label="PWSTR"), "SenderDept": SimTypeBottom(label="PWSTR"), "BillingCode": SimTypeBottom(label="PWSTR"), "ScheduleAction": SimTypeInt(signed=False, label="UInt32"), "ScheduleTime": SimTypeBottom(label="SYSTEMTIME"), "DeliveryReportType": SimTypeInt(signed=False, label="UInt32"), "DeliveryReportAddress": SimTypeBottom(label="PWSTR"), "DocumentName": SimTypeBottom(label="PWSTR"), "CallHandle": SimTypeInt(signed=False, label="UInt32"), "Reserved": SimTypeFixedSizeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)}, name="FAX_JOB_PARAMW", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "CoverPageName": SimTypeBottom(label="PWSTR"), "UseServerCoverPage": SimTypeBottom(label="BOOL"), "RecName": SimTypeBottom(label="PWSTR"), "RecFaxNumber": SimTypeBottom(label="PWSTR"), "RecCompany": SimTypeBottom(label="PWSTR"), "RecStreetAddress": SimTypeBottom(label="PWSTR"), "RecCity": SimTypeBottom(label="PWSTR"), "RecState": SimTypeBottom(label="PWSTR"), "RecZip": SimTypeBottom(label="PWSTR"), "RecCountry": SimTypeBottom(label="PWSTR"), "RecTitle": SimTypeBottom(label="PWSTR"), "RecDepartment": SimTypeBottom(label="PWSTR"), "RecOfficeLocation": SimTypeBottom(label="PWSTR"), "RecHomePhone": SimTypeBottom(label="PWSTR"), "RecOfficePhone": SimTypeBottom(label="PWSTR"), "SdrName": SimTypeBottom(label="PWSTR"), "SdrFaxNumber": SimTypeBottom(label="PWSTR"), "SdrCompany": SimTypeBottom(label="PWSTR"), "SdrAddress": SimTypeBottom(label="PWSTR"), "SdrTitle": SimTypeBottom(label="PWSTR"), "SdrDepartment": SimTypeBottom(label="PWSTR"), "SdrOfficeLocation": SimTypeBottom(label="PWSTR"), "SdrHomePhone": SimTypeBottom(label="PWSTR"), "SdrOfficePhone": SimTypeBottom(label="PWSTR"), "Note": SimTypeBottom(label="PWSTR"), "Subject": SimTypeBottom(label="PWSTR"), "TimeSent": SimTypeBottom(label="SYSTEMTIME"), "PageCount": SimTypeInt(signed=False, label="UInt32")}, name="FAX_COVERPAGE_INFOW", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["JobParams", "CoverpageInfo"]),
        # 
        'FaxSendDocumentA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "RecipientNumber": SimTypeBottom(label="PSTR"), "RecipientName": SimTypeBottom(label="PSTR"), "Tsid": SimTypeBottom(label="PSTR"), "SenderName": SimTypeBottom(label="PSTR"), "SenderCompany": SimTypeBottom(label="PSTR"), "SenderDept": SimTypeBottom(label="PSTR"), "BillingCode": SimTypeBottom(label="PSTR"), "ScheduleAction": SimTypeInt(signed=False, label="UInt32"), "ScheduleTime": SimTypeBottom(label="SYSTEMTIME"), "DeliveryReportType": SimTypeInt(signed=False, label="UInt32"), "DeliveryReportAddress": SimTypeBottom(label="PSTR"), "DocumentName": SimTypeBottom(label="PSTR"), "CallHandle": SimTypeInt(signed=False, label="UInt32"), "Reserved": SimTypeFixedSizeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)}, name="FAX_JOB_PARAMA", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "CoverPageName": SimTypeBottom(label="PSTR"), "UseServerCoverPage": SimTypeBottom(label="BOOL"), "RecName": SimTypeBottom(label="PSTR"), "RecFaxNumber": SimTypeBottom(label="PSTR"), "RecCompany": SimTypeBottom(label="PSTR"), "RecStreetAddress": SimTypeBottom(label="PSTR"), "RecCity": SimTypeBottom(label="PSTR"), "RecState": SimTypeBottom(label="PSTR"), "RecZip": SimTypeBottom(label="PSTR"), "RecCountry": SimTypeBottom(label="PSTR"), "RecTitle": SimTypeBottom(label="PSTR"), "RecDepartment": SimTypeBottom(label="PSTR"), "RecOfficeLocation": SimTypeBottom(label="PSTR"), "RecHomePhone": SimTypeBottom(label="PSTR"), "RecOfficePhone": SimTypeBottom(label="PSTR"), "SdrName": SimTypeBottom(label="PSTR"), "SdrFaxNumber": SimTypeBottom(label="PSTR"), "SdrCompany": SimTypeBottom(label="PSTR"), "SdrAddress": SimTypeBottom(label="PSTR"), "SdrTitle": SimTypeBottom(label="PSTR"), "SdrDepartment": SimTypeBottom(label="PSTR"), "SdrOfficeLocation": SimTypeBottom(label="PSTR"), "SdrHomePhone": SimTypeBottom(label="PSTR"), "SdrOfficePhone": SimTypeBottom(label="PSTR"), "Note": SimTypeBottom(label="PSTR"), "Subject": SimTypeBottom(label="PSTR"), "TimeSent": SimTypeBottom(label="SYSTEMTIME"), "PageCount": SimTypeInt(signed=False, label="UInt32")}, name="FAX_COVERPAGE_INFOA", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FileName", "JobParams", "CoverpageInfo", "FaxJobId"]),
        # 
        'FaxSendDocumentW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "RecipientNumber": SimTypeBottom(label="PWSTR"), "RecipientName": SimTypeBottom(label="PWSTR"), "Tsid": SimTypeBottom(label="PWSTR"), "SenderName": SimTypeBottom(label="PWSTR"), "SenderCompany": SimTypeBottom(label="PWSTR"), "SenderDept": SimTypeBottom(label="PWSTR"), "BillingCode": SimTypeBottom(label="PWSTR"), "ScheduleAction": SimTypeInt(signed=False, label="UInt32"), "ScheduleTime": SimTypeBottom(label="SYSTEMTIME"), "DeliveryReportType": SimTypeInt(signed=False, label="UInt32"), "DeliveryReportAddress": SimTypeBottom(label="PWSTR"), "DocumentName": SimTypeBottom(label="PWSTR"), "CallHandle": SimTypeInt(signed=False, label="UInt32"), "Reserved": SimTypeFixedSizeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)}, name="FAX_JOB_PARAMW", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "CoverPageName": SimTypeBottom(label="PWSTR"), "UseServerCoverPage": SimTypeBottom(label="BOOL"), "RecName": SimTypeBottom(label="PWSTR"), "RecFaxNumber": SimTypeBottom(label="PWSTR"), "RecCompany": SimTypeBottom(label="PWSTR"), "RecStreetAddress": SimTypeBottom(label="PWSTR"), "RecCity": SimTypeBottom(label="PWSTR"), "RecState": SimTypeBottom(label="PWSTR"), "RecZip": SimTypeBottom(label="PWSTR"), "RecCountry": SimTypeBottom(label="PWSTR"), "RecTitle": SimTypeBottom(label="PWSTR"), "RecDepartment": SimTypeBottom(label="PWSTR"), "RecOfficeLocation": SimTypeBottom(label="PWSTR"), "RecHomePhone": SimTypeBottom(label="PWSTR"), "RecOfficePhone": SimTypeBottom(label="PWSTR"), "SdrName": SimTypeBottom(label="PWSTR"), "SdrFaxNumber": SimTypeBottom(label="PWSTR"), "SdrCompany": SimTypeBottom(label="PWSTR"), "SdrAddress": SimTypeBottom(label="PWSTR"), "SdrTitle": SimTypeBottom(label="PWSTR"), "SdrDepartment": SimTypeBottom(label="PWSTR"), "SdrOfficeLocation": SimTypeBottom(label="PWSTR"), "SdrHomePhone": SimTypeBottom(label="PWSTR"), "SdrOfficePhone": SimTypeBottom(label="PWSTR"), "Note": SimTypeBottom(label="PWSTR"), "Subject": SimTypeBottom(label="PWSTR"), "TimeSent": SimTypeBottom(label="SYSTEMTIME"), "PageCount": SimTypeInt(signed=False, label="UInt32")}, name="FAX_COVERPAGE_INFOW", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FileName", "JobParams", "CoverpageInfo", "FaxJobId"]),
        # 
        'FaxSendDocumentForBroadcastA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeFunction([SimTypeBottom(label="HANDLE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "RecipientNumber": SimTypeBottom(label="PSTR"), "RecipientName": SimTypeBottom(label="PSTR"), "Tsid": SimTypeBottom(label="PSTR"), "SenderName": SimTypeBottom(label="PSTR"), "SenderCompany": SimTypeBottom(label="PSTR"), "SenderDept": SimTypeBottom(label="PSTR"), "BillingCode": SimTypeBottom(label="PSTR"), "ScheduleAction": SimTypeInt(signed=False, label="UInt32"), "ScheduleTime": SimTypeBottom(label="SYSTEMTIME"), "DeliveryReportType": SimTypeInt(signed=False, label="UInt32"), "DeliveryReportAddress": SimTypeBottom(label="PSTR"), "DocumentName": SimTypeBottom(label="PSTR"), "CallHandle": SimTypeInt(signed=False, label="UInt32"), "Reserved": SimTypeFixedSizeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)}, name="FAX_JOB_PARAMA", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "CoverPageName": SimTypeBottom(label="PSTR"), "UseServerCoverPage": SimTypeBottom(label="BOOL"), "RecName": SimTypeBottom(label="PSTR"), "RecFaxNumber": SimTypeBottom(label="PSTR"), "RecCompany": SimTypeBottom(label="PSTR"), "RecStreetAddress": SimTypeBottom(label="PSTR"), "RecCity": SimTypeBottom(label="PSTR"), "RecState": SimTypeBottom(label="PSTR"), "RecZip": SimTypeBottom(label="PSTR"), "RecCountry": SimTypeBottom(label="PSTR"), "RecTitle": SimTypeBottom(label="PSTR"), "RecDepartment": SimTypeBottom(label="PSTR"), "RecOfficeLocation": SimTypeBottom(label="PSTR"), "RecHomePhone": SimTypeBottom(label="PSTR"), "RecOfficePhone": SimTypeBottom(label="PSTR"), "SdrName": SimTypeBottom(label="PSTR"), "SdrFaxNumber": SimTypeBottom(label="PSTR"), "SdrCompany": SimTypeBottom(label="PSTR"), "SdrAddress": SimTypeBottom(label="PSTR"), "SdrTitle": SimTypeBottom(label="PSTR"), "SdrDepartment": SimTypeBottom(label="PSTR"), "SdrOfficeLocation": SimTypeBottom(label="PSTR"), "SdrHomePhone": SimTypeBottom(label="PSTR"), "SdrOfficePhone": SimTypeBottom(label="PSTR"), "Note": SimTypeBottom(label="PSTR"), "Subject": SimTypeBottom(label="PSTR"), "TimeSent": SimTypeBottom(label="SYSTEMTIME"), "PageCount": SimTypeInt(signed=False, label="UInt32")}, name="FAX_COVERPAGE_INFOA", pack=False, align=None), offset=0)], SimTypeBottom(label="BOOL"), arg_names=["FaxHandle", "RecipientNumber", "Context", "JobParams", "CoverpageInfo"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FileName", "FaxJobId", "FaxRecipientCallback", "Context"]),
        # 
        'FaxSendDocumentForBroadcastW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeFunction([SimTypeBottom(label="HANDLE"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "RecipientNumber": SimTypeBottom(label="PWSTR"), "RecipientName": SimTypeBottom(label="PWSTR"), "Tsid": SimTypeBottom(label="PWSTR"), "SenderName": SimTypeBottom(label="PWSTR"), "SenderCompany": SimTypeBottom(label="PWSTR"), "SenderDept": SimTypeBottom(label="PWSTR"), "BillingCode": SimTypeBottom(label="PWSTR"), "ScheduleAction": SimTypeInt(signed=False, label="UInt32"), "ScheduleTime": SimTypeBottom(label="SYSTEMTIME"), "DeliveryReportType": SimTypeInt(signed=False, label="UInt32"), "DeliveryReportAddress": SimTypeBottom(label="PWSTR"), "DocumentName": SimTypeBottom(label="PWSTR"), "CallHandle": SimTypeInt(signed=False, label="UInt32"), "Reserved": SimTypeFixedSizeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 3)}, name="FAX_JOB_PARAMW", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "CoverPageName": SimTypeBottom(label="PWSTR"), "UseServerCoverPage": SimTypeBottom(label="BOOL"), "RecName": SimTypeBottom(label="PWSTR"), "RecFaxNumber": SimTypeBottom(label="PWSTR"), "RecCompany": SimTypeBottom(label="PWSTR"), "RecStreetAddress": SimTypeBottom(label="PWSTR"), "RecCity": SimTypeBottom(label="PWSTR"), "RecState": SimTypeBottom(label="PWSTR"), "RecZip": SimTypeBottom(label="PWSTR"), "RecCountry": SimTypeBottom(label="PWSTR"), "RecTitle": SimTypeBottom(label="PWSTR"), "RecDepartment": SimTypeBottom(label="PWSTR"), "RecOfficeLocation": SimTypeBottom(label="PWSTR"), "RecHomePhone": SimTypeBottom(label="PWSTR"), "RecOfficePhone": SimTypeBottom(label="PWSTR"), "SdrName": SimTypeBottom(label="PWSTR"), "SdrFaxNumber": SimTypeBottom(label="PWSTR"), "SdrCompany": SimTypeBottom(label="PWSTR"), "SdrAddress": SimTypeBottom(label="PWSTR"), "SdrTitle": SimTypeBottom(label="PWSTR"), "SdrDepartment": SimTypeBottom(label="PWSTR"), "SdrOfficeLocation": SimTypeBottom(label="PWSTR"), "SdrHomePhone": SimTypeBottom(label="PWSTR"), "SdrOfficePhone": SimTypeBottom(label="PWSTR"), "Note": SimTypeBottom(label="PWSTR"), "Subject": SimTypeBottom(label="PWSTR"), "TimeSent": SimTypeBottom(label="SYSTEMTIME"), "PageCount": SimTypeInt(signed=False, label="UInt32")}, name="FAX_COVERPAGE_INFOW", pack=False, align=None), offset=0)], SimTypeBottom(label="BOOL"), arg_names=["FaxHandle", "RecipientNumber", "Context", "JobParams", "CoverpageInfo"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FileName", "FaxJobId", "FaxRecipientCallback", "Context"]),
        # 
        'FaxEnumJobsA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "JobId": SimTypeInt(signed=False, label="UInt32"), "UserName": SimTypeBottom(label="PSTR"), "JobType": SimTypeInt(signed=False, label="UInt32"), "QueueStatus": SimTypeInt(signed=False, label="UInt32"), "Status": SimTypeInt(signed=False, label="UInt32"), "Size": SimTypeInt(signed=False, label="UInt32"), "PageCount": SimTypeInt(signed=False, label="UInt32"), "RecipientNumber": SimTypeBottom(label="PSTR"), "RecipientName": SimTypeBottom(label="PSTR"), "Tsid": SimTypeBottom(label="PSTR"), "SenderName": SimTypeBottom(label="PSTR"), "SenderCompany": SimTypeBottom(label="PSTR"), "SenderDept": SimTypeBottom(label="PSTR"), "BillingCode": SimTypeBottom(label="PSTR"), "ScheduleAction": SimTypeInt(signed=False, label="UInt32"), "ScheduleTime": SimTypeBottom(label="SYSTEMTIME"), "DeliveryReportType": SimTypeInt(signed=False, label="UInt32"), "DeliveryReportAddress": SimTypeBottom(label="PSTR"), "DocumentName": SimTypeBottom(label="PSTR")}, name="FAX_JOB_ENTRYA", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobEntry", "JobsReturned"]),
        # 
        'FaxEnumJobsW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "JobId": SimTypeInt(signed=False, label="UInt32"), "UserName": SimTypeBottom(label="PWSTR"), "JobType": SimTypeInt(signed=False, label="UInt32"), "QueueStatus": SimTypeInt(signed=False, label="UInt32"), "Status": SimTypeInt(signed=False, label="UInt32"), "Size": SimTypeInt(signed=False, label="UInt32"), "PageCount": SimTypeInt(signed=False, label="UInt32"), "RecipientNumber": SimTypeBottom(label="PWSTR"), "RecipientName": SimTypeBottom(label="PWSTR"), "Tsid": SimTypeBottom(label="PWSTR"), "SenderName": SimTypeBottom(label="PWSTR"), "SenderCompany": SimTypeBottom(label="PWSTR"), "SenderDept": SimTypeBottom(label="PWSTR"), "BillingCode": SimTypeBottom(label="PWSTR"), "ScheduleAction": SimTypeInt(signed=False, label="UInt32"), "ScheduleTime": SimTypeBottom(label="SYSTEMTIME"), "DeliveryReportType": SimTypeInt(signed=False, label="UInt32"), "DeliveryReportAddress": SimTypeBottom(label="PWSTR"), "DocumentName": SimTypeBottom(label="PWSTR")}, name="FAX_JOB_ENTRYW", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobEntry", "JobsReturned"]),
        # 
        'FaxGetJobA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "JobId": SimTypeInt(signed=False, label="UInt32"), "UserName": SimTypeBottom(label="PSTR"), "JobType": SimTypeInt(signed=False, label="UInt32"), "QueueStatus": SimTypeInt(signed=False, label="UInt32"), "Status": SimTypeInt(signed=False, label="UInt32"), "Size": SimTypeInt(signed=False, label="UInt32"), "PageCount": SimTypeInt(signed=False, label="UInt32"), "RecipientNumber": SimTypeBottom(label="PSTR"), "RecipientName": SimTypeBottom(label="PSTR"), "Tsid": SimTypeBottom(label="PSTR"), "SenderName": SimTypeBottom(label="PSTR"), "SenderCompany": SimTypeBottom(label="PSTR"), "SenderDept": SimTypeBottom(label="PSTR"), "BillingCode": SimTypeBottom(label="PSTR"), "ScheduleAction": SimTypeInt(signed=False, label="UInt32"), "ScheduleTime": SimTypeBottom(label="SYSTEMTIME"), "DeliveryReportType": SimTypeInt(signed=False, label="UInt32"), "DeliveryReportAddress": SimTypeBottom(label="PSTR"), "DocumentName": SimTypeBottom(label="PSTR")}, name="FAX_JOB_ENTRYA", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobId", "JobEntry"]),
        # 
        'FaxGetJobW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "JobId": SimTypeInt(signed=False, label="UInt32"), "UserName": SimTypeBottom(label="PWSTR"), "JobType": SimTypeInt(signed=False, label="UInt32"), "QueueStatus": SimTypeInt(signed=False, label="UInt32"), "Status": SimTypeInt(signed=False, label="UInt32"), "Size": SimTypeInt(signed=False, label="UInt32"), "PageCount": SimTypeInt(signed=False, label="UInt32"), "RecipientNumber": SimTypeBottom(label="PWSTR"), "RecipientName": SimTypeBottom(label="PWSTR"), "Tsid": SimTypeBottom(label="PWSTR"), "SenderName": SimTypeBottom(label="PWSTR"), "SenderCompany": SimTypeBottom(label="PWSTR"), "SenderDept": SimTypeBottom(label="PWSTR"), "BillingCode": SimTypeBottom(label="PWSTR"), "ScheduleAction": SimTypeInt(signed=False, label="UInt32"), "ScheduleTime": SimTypeBottom(label="SYSTEMTIME"), "DeliveryReportType": SimTypeInt(signed=False, label="UInt32"), "DeliveryReportAddress": SimTypeBottom(label="PWSTR"), "DocumentName": SimTypeBottom(label="PWSTR")}, name="FAX_JOB_ENTRYW", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobId", "JobEntry"]),
        # 
        'FaxSetJobA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "JobId": SimTypeInt(signed=False, label="UInt32"), "UserName": SimTypeBottom(label="PSTR"), "JobType": SimTypeInt(signed=False, label="UInt32"), "QueueStatus": SimTypeInt(signed=False, label="UInt32"), "Status": SimTypeInt(signed=False, label="UInt32"), "Size": SimTypeInt(signed=False, label="UInt32"), "PageCount": SimTypeInt(signed=False, label="UInt32"), "RecipientNumber": SimTypeBottom(label="PSTR"), "RecipientName": SimTypeBottom(label="PSTR"), "Tsid": SimTypeBottom(label="PSTR"), "SenderName": SimTypeBottom(label="PSTR"), "SenderCompany": SimTypeBottom(label="PSTR"), "SenderDept": SimTypeBottom(label="PSTR"), "BillingCode": SimTypeBottom(label="PSTR"), "ScheduleAction": SimTypeInt(signed=False, label="UInt32"), "ScheduleTime": SimTypeBottom(label="SYSTEMTIME"), "DeliveryReportType": SimTypeInt(signed=False, label="UInt32"), "DeliveryReportAddress": SimTypeBottom(label="PSTR"), "DocumentName": SimTypeBottom(label="PSTR")}, name="FAX_JOB_ENTRYA", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobId", "Command", "JobEntry"]),
        # 
        'FaxSetJobW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "JobId": SimTypeInt(signed=False, label="UInt32"), "UserName": SimTypeBottom(label="PWSTR"), "JobType": SimTypeInt(signed=False, label="UInt32"), "QueueStatus": SimTypeInt(signed=False, label="UInt32"), "Status": SimTypeInt(signed=False, label="UInt32"), "Size": SimTypeInt(signed=False, label="UInt32"), "PageCount": SimTypeInt(signed=False, label="UInt32"), "RecipientNumber": SimTypeBottom(label="PWSTR"), "RecipientName": SimTypeBottom(label="PWSTR"), "Tsid": SimTypeBottom(label="PWSTR"), "SenderName": SimTypeBottom(label="PWSTR"), "SenderCompany": SimTypeBottom(label="PWSTR"), "SenderDept": SimTypeBottom(label="PWSTR"), "BillingCode": SimTypeBottom(label="PWSTR"), "ScheduleAction": SimTypeInt(signed=False, label="UInt32"), "ScheduleTime": SimTypeBottom(label="SYSTEMTIME"), "DeliveryReportType": SimTypeInt(signed=False, label="UInt32"), "DeliveryReportAddress": SimTypeBottom(label="PWSTR"), "DocumentName": SimTypeBottom(label="PWSTR")}, name="FAX_JOB_ENTRYW", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobId", "Command", "JobEntry"]),
        # 
        'FaxGetPageData': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobId", "Buffer", "BufferSize", "ImageWidth", "ImageHeight"]),
        # 
        'FaxGetDeviceStatusA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "CallerId": SimTypeBottom(label="PSTR"), "Csid": SimTypeBottom(label="PSTR"), "CurrentPage": SimTypeInt(signed=False, label="UInt32"), "DeviceId": SimTypeInt(signed=False, label="UInt32"), "DeviceName": SimTypeBottom(label="PSTR"), "DocumentName": SimTypeBottom(label="PSTR"), "JobType": SimTypeInt(signed=False, label="UInt32"), "PhoneNumber": SimTypeBottom(label="PSTR"), "RoutingString": SimTypeBottom(label="PSTR"), "SenderName": SimTypeBottom(label="PSTR"), "RecipientName": SimTypeBottom(label="PSTR"), "Size": SimTypeInt(signed=False, label="UInt32"), "StartTime": SimTypeBottom(label="FILETIME"), "Status": SimTypeInt(signed=False, label="UInt32"), "StatusString": SimTypeBottom(label="PSTR"), "SubmittedTime": SimTypeBottom(label="FILETIME"), "TotalPages": SimTypeInt(signed=False, label="UInt32"), "Tsid": SimTypeBottom(label="PSTR"), "UserName": SimTypeBottom(label="PSTR")}, name="FAX_DEVICE_STATUSA", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "DeviceStatus"]),
        # 
        'FaxGetDeviceStatusW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "CallerId": SimTypeBottom(label="PWSTR"), "Csid": SimTypeBottom(label="PWSTR"), "CurrentPage": SimTypeInt(signed=False, label="UInt32"), "DeviceId": SimTypeInt(signed=False, label="UInt32"), "DeviceName": SimTypeBottom(label="PWSTR"), "DocumentName": SimTypeBottom(label="PWSTR"), "JobType": SimTypeInt(signed=False, label="UInt32"), "PhoneNumber": SimTypeBottom(label="PWSTR"), "RoutingString": SimTypeBottom(label="PWSTR"), "SenderName": SimTypeBottom(label="PWSTR"), "RecipientName": SimTypeBottom(label="PWSTR"), "Size": SimTypeInt(signed=False, label="UInt32"), "StartTime": SimTypeBottom(label="FILETIME"), "Status": SimTypeInt(signed=False, label="UInt32"), "StatusString": SimTypeBottom(label="PWSTR"), "SubmittedTime": SimTypeBottom(label="FILETIME"), "TotalPages": SimTypeInt(signed=False, label="UInt32"), "Tsid": SimTypeBottom(label="PWSTR"), "UserName": SimTypeBottom(label="PWSTR")}, name="FAX_DEVICE_STATUSW", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "DeviceStatus"]),
        # 
        'FaxAbort': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "JobId"]),
        # 
        'FaxGetConfigurationA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "Retries": SimTypeInt(signed=False, label="UInt32"), "RetryDelay": SimTypeInt(signed=False, label="UInt32"), "DirtyDays": SimTypeInt(signed=False, label="UInt32"), "Branding": SimTypeBottom(label="BOOL"), "UseDeviceTsid": SimTypeBottom(label="BOOL"), "ServerCp": SimTypeBottom(label="BOOL"), "PauseServerQueue": SimTypeBottom(label="BOOL"), "StartCheapTime": SimStruct({"Hour": SimTypeShort(signed=False, label="UInt16"), "Minute": SimTypeShort(signed=False, label="UInt16")}, name="FAX_TIME", pack=False, align=None), "StopCheapTime": SimStruct({"Hour": SimTypeShort(signed=False, label="UInt16"), "Minute": SimTypeShort(signed=False, label="UInt16")}, name="FAX_TIME", pack=False, align=None), "ArchiveOutgoingFaxes": SimTypeBottom(label="BOOL"), "ArchiveDirectory": SimTypeBottom(label="PSTR"), "Reserved": SimTypeBottom(label="PSTR")}, name="FAX_CONFIGURATIONA", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FaxConfig"]),
        # 
        'FaxGetConfigurationW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "Retries": SimTypeInt(signed=False, label="UInt32"), "RetryDelay": SimTypeInt(signed=False, label="UInt32"), "DirtyDays": SimTypeInt(signed=False, label="UInt32"), "Branding": SimTypeBottom(label="BOOL"), "UseDeviceTsid": SimTypeBottom(label="BOOL"), "ServerCp": SimTypeBottom(label="BOOL"), "PauseServerQueue": SimTypeBottom(label="BOOL"), "StartCheapTime": SimStruct({"Hour": SimTypeShort(signed=False, label="UInt16"), "Minute": SimTypeShort(signed=False, label="UInt16")}, name="FAX_TIME", pack=False, align=None), "StopCheapTime": SimStruct({"Hour": SimTypeShort(signed=False, label="UInt16"), "Minute": SimTypeShort(signed=False, label="UInt16")}, name="FAX_TIME", pack=False, align=None), "ArchiveOutgoingFaxes": SimTypeBottom(label="BOOL"), "ArchiveDirectory": SimTypeBottom(label="PWSTR"), "Reserved": SimTypeBottom(label="PWSTR")}, name="FAX_CONFIGURATIONW", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FaxConfig"]),
        # 
        'FaxSetConfigurationA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "Retries": SimTypeInt(signed=False, label="UInt32"), "RetryDelay": SimTypeInt(signed=False, label="UInt32"), "DirtyDays": SimTypeInt(signed=False, label="UInt32"), "Branding": SimTypeBottom(label="BOOL"), "UseDeviceTsid": SimTypeBottom(label="BOOL"), "ServerCp": SimTypeBottom(label="BOOL"), "PauseServerQueue": SimTypeBottom(label="BOOL"), "StartCheapTime": SimStruct({"Hour": SimTypeShort(signed=False, label="UInt16"), "Minute": SimTypeShort(signed=False, label="UInt16")}, name="FAX_TIME", pack=False, align=None), "StopCheapTime": SimStruct({"Hour": SimTypeShort(signed=False, label="UInt16"), "Minute": SimTypeShort(signed=False, label="UInt16")}, name="FAX_TIME", pack=False, align=None), "ArchiveOutgoingFaxes": SimTypeBottom(label="BOOL"), "ArchiveDirectory": SimTypeBottom(label="PSTR"), "Reserved": SimTypeBottom(label="PSTR")}, name="FAX_CONFIGURATIONA", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FaxConfig"]),
        # 
        'FaxSetConfigurationW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "Retries": SimTypeInt(signed=False, label="UInt32"), "RetryDelay": SimTypeInt(signed=False, label="UInt32"), "DirtyDays": SimTypeInt(signed=False, label="UInt32"), "Branding": SimTypeBottom(label="BOOL"), "UseDeviceTsid": SimTypeBottom(label="BOOL"), "ServerCp": SimTypeBottom(label="BOOL"), "PauseServerQueue": SimTypeBottom(label="BOOL"), "StartCheapTime": SimStruct({"Hour": SimTypeShort(signed=False, label="UInt16"), "Minute": SimTypeShort(signed=False, label="UInt16")}, name="FAX_TIME", pack=False, align=None), "StopCheapTime": SimStruct({"Hour": SimTypeShort(signed=False, label="UInt16"), "Minute": SimTypeShort(signed=False, label="UInt16")}, name="FAX_TIME", pack=False, align=None), "ArchiveOutgoingFaxes": SimTypeBottom(label="BOOL"), "ArchiveDirectory": SimTypeBottom(label="PWSTR"), "Reserved": SimTypeBottom(label="PWSTR")}, name="FAX_CONFIGURATIONW", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "FaxConfig"]),
        # 
        'FaxGetLoggingCategoriesA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"Name": SimTypeBottom(label="PSTR"), "Category": SimTypeInt(signed=False, label="UInt32"), "Level": SimTypeInt(signed=False, label="UInt32")}, name="FAX_LOG_CATEGORYA", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "Categories", "NumberCategories"]),
        # 
        'FaxGetLoggingCategoriesW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"Name": SimTypeBottom(label="PWSTR"), "Category": SimTypeInt(signed=False, label="UInt32"), "Level": SimTypeInt(signed=False, label="UInt32")}, name="FAX_LOG_CATEGORYW", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "Categories", "NumberCategories"]),
        # 
        'FaxSetLoggingCategoriesA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"Name": SimTypeBottom(label="PSTR"), "Category": SimTypeInt(signed=False, label="UInt32"), "Level": SimTypeInt(signed=False, label="UInt32")}, name="FAX_LOG_CATEGORYA", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "Categories", "NumberCategories"]),
        # 
        'FaxSetLoggingCategoriesW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"Name": SimTypeBottom(label="PWSTR"), "Category": SimTypeInt(signed=False, label="UInt32"), "Level": SimTypeInt(signed=False, label="UInt32")}, name="FAX_LOG_CATEGORYW", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "Categories", "NumberCategories"]),
        # 
        'FaxEnumPortsA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "DeviceId": SimTypeInt(signed=False, label="UInt32"), "State": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="UInt32"), "Rings": SimTypeInt(signed=False, label="UInt32"), "Priority": SimTypeInt(signed=False, label="UInt32"), "DeviceName": SimTypeBottom(label="PSTR"), "Tsid": SimTypeBottom(label="PSTR"), "Csid": SimTypeBottom(label="PSTR")}, name="FAX_PORT_INFOA", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "PortInfo", "PortsReturned"]),
        # 
        'FaxEnumPortsW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "DeviceId": SimTypeInt(signed=False, label="UInt32"), "State": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="UInt32"), "Rings": SimTypeInt(signed=False, label="UInt32"), "Priority": SimTypeInt(signed=False, label="UInt32"), "DeviceName": SimTypeBottom(label="PWSTR"), "Tsid": SimTypeBottom(label="PWSTR"), "Csid": SimTypeBottom(label="PWSTR")}, name="FAX_PORT_INFOW", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "PortInfo", "PortsReturned"]),
        # 
        'FaxGetPortA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "DeviceId": SimTypeInt(signed=False, label="UInt32"), "State": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="UInt32"), "Rings": SimTypeInt(signed=False, label="UInt32"), "Priority": SimTypeInt(signed=False, label="UInt32"), "DeviceName": SimTypeBottom(label="PSTR"), "Tsid": SimTypeBottom(label="PSTR"), "Csid": SimTypeBottom(label="PSTR")}, name="FAX_PORT_INFOA", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "PortInfo"]),
        # 
        'FaxGetPortW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "DeviceId": SimTypeInt(signed=False, label="UInt32"), "State": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="UInt32"), "Rings": SimTypeInt(signed=False, label="UInt32"), "Priority": SimTypeInt(signed=False, label="UInt32"), "DeviceName": SimTypeBottom(label="PWSTR"), "Tsid": SimTypeBottom(label="PWSTR"), "Csid": SimTypeBottom(label="PWSTR")}, name="FAX_PORT_INFOW", pack=False, align=None), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "PortInfo"]),
        # 
        'FaxSetPortA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "DeviceId": SimTypeInt(signed=False, label="UInt32"), "State": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="UInt32"), "Rings": SimTypeInt(signed=False, label="UInt32"), "Priority": SimTypeInt(signed=False, label="UInt32"), "DeviceName": SimTypeBottom(label="PSTR"), "Tsid": SimTypeBottom(label="PSTR"), "Csid": SimTypeBottom(label="PSTR")}, name="FAX_PORT_INFOA", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "PortInfo"]),
        # 
        'FaxSetPortW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "DeviceId": SimTypeInt(signed=False, label="UInt32"), "State": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="UInt32"), "Rings": SimTypeInt(signed=False, label="UInt32"), "Priority": SimTypeInt(signed=False, label="UInt32"), "DeviceName": SimTypeBottom(label="PWSTR"), "Tsid": SimTypeBottom(label="PWSTR"), "Csid": SimTypeBottom(label="PWSTR")}, name="FAX_PORT_INFOW", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "PortInfo"]),
        # 
        'FaxEnumRoutingMethodsA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "DeviceId": SimTypeInt(signed=False, label="UInt32"), "Enabled": SimTypeBottom(label="BOOL"), "DeviceName": SimTypeBottom(label="PSTR"), "Guid": SimTypeBottom(label="PSTR"), "FriendlyName": SimTypeBottom(label="PSTR"), "FunctionName": SimTypeBottom(label="PSTR"), "ExtensionImageName": SimTypeBottom(label="PSTR"), "ExtensionFriendlyName": SimTypeBottom(label="PSTR")}, name="FAX_ROUTING_METHODA", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingMethod", "MethodsReturned"]),
        # 
        'FaxEnumRoutingMethodsW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "DeviceId": SimTypeInt(signed=False, label="UInt32"), "Enabled": SimTypeBottom(label="BOOL"), "DeviceName": SimTypeBottom(label="PWSTR"), "Guid": SimTypeBottom(label="PWSTR"), "FriendlyName": SimTypeBottom(label="PWSTR"), "FunctionName": SimTypeBottom(label="PWSTR"), "ExtensionImageName": SimTypeBottom(label="PWSTR"), "ExtensionFriendlyName": SimTypeBottom(label="PWSTR")}, name="FAX_ROUTING_METHODW", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingMethod", "MethodsReturned"]),
        # 
        'FaxEnableRoutingMethodA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingGuid", "Enabled"]),
        # 
        'FaxEnableRoutingMethodW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingGuid", "Enabled"]),
        # 
        'FaxEnumGlobalRoutingInfoA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "Priority": SimTypeInt(signed=False, label="UInt32"), "Guid": SimTypeBottom(label="PSTR"), "FriendlyName": SimTypeBottom(label="PSTR"), "FunctionName": SimTypeBottom(label="PSTR"), "ExtensionImageName": SimTypeBottom(label="PSTR"), "ExtensionFriendlyName": SimTypeBottom(label="PSTR")}, name="FAX_GLOBAL_ROUTING_INFOA", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "RoutingInfo", "MethodsReturned"]),
        # 
        'FaxEnumGlobalRoutingInfoW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "Priority": SimTypeInt(signed=False, label="UInt32"), "Guid": SimTypeBottom(label="PWSTR"), "FriendlyName": SimTypeBottom(label="PWSTR"), "FunctionName": SimTypeBottom(label="PWSTR"), "ExtensionImageName": SimTypeBottom(label="PWSTR"), "ExtensionFriendlyName": SimTypeBottom(label="PWSTR")}, name="FAX_GLOBAL_ROUTING_INFOW", pack=False, align=None), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "RoutingInfo", "MethodsReturned"]),
        # 
        'FaxSetGlobalRoutingInfoA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "Priority": SimTypeInt(signed=False, label="UInt32"), "Guid": SimTypeBottom(label="PSTR"), "FriendlyName": SimTypeBottom(label="PSTR"), "FunctionName": SimTypeBottom(label="PSTR"), "ExtensionImageName": SimTypeBottom(label="PSTR"), "ExtensionFriendlyName": SimTypeBottom(label="PSTR")}, name="FAX_GLOBAL_ROUTING_INFOA", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "RoutingInfo"]),
        # 
        'FaxSetGlobalRoutingInfoW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "Priority": SimTypeInt(signed=False, label="UInt32"), "Guid": SimTypeBottom(label="PWSTR"), "FriendlyName": SimTypeBottom(label="PWSTR"), "FunctionName": SimTypeBottom(label="PWSTR"), "ExtensionImageName": SimTypeBottom(label="PWSTR"), "ExtensionFriendlyName": SimTypeBottom(label="PWSTR")}, name="FAX_GLOBAL_ROUTING_INFOW", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "RoutingInfo"]),
        # 
        'FaxGetRoutingInfoA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingGuid", "RoutingInfoBuffer", "RoutingInfoBufferSize"]),
        # 
        'FaxGetRoutingInfoW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Byte"), offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingGuid", "RoutingInfoBuffer", "RoutingInfoBufferSize"]),
        # 
        'FaxSetRoutingInfoA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingGuid", "RoutingInfoBuffer", "RoutingInfoBufferSize"]),
        # 
        'FaxSetRoutingInfoW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxPortHandle", "RoutingGuid", "RoutingInfoBuffer", "RoutingInfoBufferSize"]),
        # 
        'FaxInitializeEventQueue': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "CompletionPort", "CompletionKey", "hWnd", "MessageStart"]),
        # 
        'FaxFreeBuffer': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["Buffer"]),
        # 
        'FaxStartPrintJobA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "DocName": SimTypeBottom(label="PSTR"), "RecipientName": SimTypeBottom(label="PSTR"), "RecipientNumber": SimTypeBottom(label="PSTR"), "SenderName": SimTypeBottom(label="PSTR"), "SenderCompany": SimTypeBottom(label="PSTR"), "SenderDept": SimTypeBottom(label="PSTR"), "SenderBillingCode": SimTypeBottom(label="PSTR"), "Reserved": SimTypeBottom(label="PSTR"), "DrEmailAddress": SimTypeBottom(label="PSTR"), "OutputFileName": SimTypeBottom(label="PSTR")}, name="FAX_PRINT_INFOA", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "hDC": SimTypeBottom(label="HDC"), "ServerName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 16)}, name="FAX_CONTEXT_INFOA", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PrinterName", "PrintInfo", "FaxJobId", "FaxContextInfo"]),
        # 
        'FaxStartPrintJobW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "DocName": SimTypeBottom(label="PWSTR"), "RecipientName": SimTypeBottom(label="PWSTR"), "RecipientNumber": SimTypeBottom(label="PWSTR"), "SenderName": SimTypeBottom(label="PWSTR"), "SenderCompany": SimTypeBottom(label="PWSTR"), "SenderDept": SimTypeBottom(label="PWSTR"), "SenderBillingCode": SimTypeBottom(label="PWSTR"), "Reserved": SimTypeBottom(label="PWSTR"), "DrEmailAddress": SimTypeBottom(label="PWSTR"), "OutputFileName": SimTypeBottom(label="PWSTR")}, name="FAX_PRINT_INFOW", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "hDC": SimTypeBottom(label="HDC"), "ServerName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 16)}, name="FAX_CONTEXT_INFOW", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PrinterName", "PrintInfo", "FaxJobId", "FaxContextInfo"]),
        # 
        'FaxPrintCoverPageA': SimTypeFunction([SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "hDC": SimTypeBottom(label="HDC"), "ServerName": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 16)}, name="FAX_CONTEXT_INFOA", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "CoverPageName": SimTypeBottom(label="PSTR"), "UseServerCoverPage": SimTypeBottom(label="BOOL"), "RecName": SimTypeBottom(label="PSTR"), "RecFaxNumber": SimTypeBottom(label="PSTR"), "RecCompany": SimTypeBottom(label="PSTR"), "RecStreetAddress": SimTypeBottom(label="PSTR"), "RecCity": SimTypeBottom(label="PSTR"), "RecState": SimTypeBottom(label="PSTR"), "RecZip": SimTypeBottom(label="PSTR"), "RecCountry": SimTypeBottom(label="PSTR"), "RecTitle": SimTypeBottom(label="PSTR"), "RecDepartment": SimTypeBottom(label="PSTR"), "RecOfficeLocation": SimTypeBottom(label="PSTR"), "RecHomePhone": SimTypeBottom(label="PSTR"), "RecOfficePhone": SimTypeBottom(label="PSTR"), "SdrName": SimTypeBottom(label="PSTR"), "SdrFaxNumber": SimTypeBottom(label="PSTR"), "SdrCompany": SimTypeBottom(label="PSTR"), "SdrAddress": SimTypeBottom(label="PSTR"), "SdrTitle": SimTypeBottom(label="PSTR"), "SdrDepartment": SimTypeBottom(label="PSTR"), "SdrOfficeLocation": SimTypeBottom(label="PSTR"), "SdrHomePhone": SimTypeBottom(label="PSTR"), "SdrOfficePhone": SimTypeBottom(label="PSTR"), "Note": SimTypeBottom(label="PSTR"), "Subject": SimTypeBottom(label="PSTR"), "TimeSent": SimTypeBottom(label="SYSTEMTIME"), "PageCount": SimTypeInt(signed=False, label="UInt32")}, name="FAX_COVERPAGE_INFOA", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxContextInfo", "CoverPageInfo"]),
        # 
        'FaxPrintCoverPageW': SimTypeFunction([SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "hDC": SimTypeBottom(label="HDC"), "ServerName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 16)}, name="FAX_CONTEXT_INFOW", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"SizeOfStruct": SimTypeInt(signed=False, label="UInt32"), "CoverPageName": SimTypeBottom(label="PWSTR"), "UseServerCoverPage": SimTypeBottom(label="BOOL"), "RecName": SimTypeBottom(label="PWSTR"), "RecFaxNumber": SimTypeBottom(label="PWSTR"), "RecCompany": SimTypeBottom(label="PWSTR"), "RecStreetAddress": SimTypeBottom(label="PWSTR"), "RecCity": SimTypeBottom(label="PWSTR"), "RecState": SimTypeBottom(label="PWSTR"), "RecZip": SimTypeBottom(label="PWSTR"), "RecCountry": SimTypeBottom(label="PWSTR"), "RecTitle": SimTypeBottom(label="PWSTR"), "RecDepartment": SimTypeBottom(label="PWSTR"), "RecOfficeLocation": SimTypeBottom(label="PWSTR"), "RecHomePhone": SimTypeBottom(label="PWSTR"), "RecOfficePhone": SimTypeBottom(label="PWSTR"), "SdrName": SimTypeBottom(label="PWSTR"), "SdrFaxNumber": SimTypeBottom(label="PWSTR"), "SdrCompany": SimTypeBottom(label="PWSTR"), "SdrAddress": SimTypeBottom(label="PWSTR"), "SdrTitle": SimTypeBottom(label="PWSTR"), "SdrDepartment": SimTypeBottom(label="PWSTR"), "SdrOfficeLocation": SimTypeBottom(label="PWSTR"), "SdrHomePhone": SimTypeBottom(label="PWSTR"), "SdrOfficePhone": SimTypeBottom(label="PWSTR"), "Note": SimTypeBottom(label="PWSTR"), "Subject": SimTypeBottom(label="PWSTR"), "TimeSent": SimTypeBottom(label="SYSTEMTIME"), "PageCount": SimTypeInt(signed=False, label="UInt32")}, name="FAX_COVERPAGE_INFOW", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxContextInfo", "CoverPageInfo"]),
        # 
        'FaxRegisterServiceProviderW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceProvider", "FriendlyName", "ImageName", "TspName"]),
        # 
        'FaxUnregisterServiceProviderW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DeviceProvider"]),
        # 
        'FaxRegisterRoutingExtensionW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeFunction([SimTypeBottom(label="HANDLE"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeBottom(label="PWSTR"), SimTypeBottom(label="PWSTR"), SimTypeBottom(label="PWSTR"), SimTypeBottom(label="PWSTR")], SimTypeBottom(label="BOOL"), arg_names=["FaxHandle", "Context", "MethodName", "FriendlyName", "FunctionName", "Guid"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "ExtensionName", "FriendlyName", "ImageName", "CallBack", "Context"]),
        # 
        'FaxAccessCheck': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["FaxHandle", "AccessMask"]),
    }

lib.set_prototypes(prototypes)



class UnknownSize:
    """
    A value indicating an unknown size for elements of DataSets.
    Should "behave" like an integer.
    """
    def __add__(self, other):
        return self

    def __radd__(self, other):
        return self

    def __sub__(self, other):
        return self

    def __rsub__(self, other):
        return self

    def __floordiv__(self, other):
        return self

    def __lshift__(self, other):
        return self

    def __rlshift__(self, other):
        return self

    def __rshift__(self, other):
        return self

    def __rrshift__(self, other):
        return self

    def __and__(self, other):
        return self

    def __rand__(self, other):
        return self

    def __xor__(self, other):
        return self

    def __rxor__(self, other):
        return self

    def __or__(self, other):
        return self

    def __ror__(self, other):
        return self

    def __neg__(self):
        return self

    def __eq__(self, other):
        return type(other) is UnknownSize

    def __ne__(self, other):
        return not (self == other)

    def __le__(self, other):
        return False

    def __lt__(self, other):
        return False

    def __ge__(self, other):
        return True

    def __gt__(self, other):
        return True

    def __hash__(self):
        return hash('unknown size')

    def __str__(self):
        return '<UnknownSize>'

    def __repr__(self):
        return "<UnknownSize>"

UNKNOWN_SIZE = UnknownSize()



###
### This file was automatically generated
###

from archinfo.arch import register_arch, Endness, Register

from .common import ArchPcode


class ArchPcode_SuperH4_LE_32_default(ArchPcode):
    name = 'SuperH4:LE:32:default'
    pcode_arch = 'SuperH4:LE:32:default'
    description = 'SuperH-4(a) (SH4) little endian'
    bits = 32
    ip_offset = 0x80c
    sp_offset = 0x3c
    bp_offset = sp_offset
    instruction_endness = Endness.LE
    register_list = [
        Register('r0', 4, 0x0),
        Register('r1', 4, 0x4),
        Register('r2', 4, 0x8),
        Register('r3', 4, 0xc),
        Register('r4', 4, 0x10),
        Register('r5', 4, 0x14),
        Register('r6', 4, 0x18),
        Register('r7', 4, 0x1c),
        Register('r8', 4, 0x20),
        Register('r9', 4, 0x24),
        Register('r10', 4, 0x28),
        Register('r11', 4, 0x2c),
        Register('r12', 4, 0x30),
        Register('r13', 4, 0x34),
        Register('r14', 4, 0x38),
        Register('r15', 4, 0x3c),
        Register('r0_bank', 4, 0x40),
        Register('r1_bank', 4, 0x44),
        Register('r2_bank', 4, 0x48),
        Register('r3_bank', 4, 0x4c),
        Register('r4_bank', 4, 0x50),
        Register('r5_bank', 4, 0x54),
        Register('r6_bank', 4, 0x58),
        Register('r7_bank', 4, 0x5c),
        Register('fv0', 16, 0x200),
        Register('dr0', 8, 0x200),
        Register('fr1', 4, 0x200),
        Register('fr0', 4, 0x204),
        Register('dr2', 8, 0x208),
        Register('fr3', 4, 0x208),
        Register('fr2', 4, 0x20c),
        Register('fv4', 16, 0x210),
        Register('dr4', 8, 0x210),
        Register('fr5', 4, 0x210),
        Register('fr4', 4, 0x214),
        Register('dr6', 8, 0x218),
        Register('fr7', 4, 0x218),
        Register('fr6', 4, 0x21c),
        Register('fv8', 16, 0x220),
        Register('dr8', 8, 0x220),
        Register('fr9', 4, 0x220),
        Register('fr8', 4, 0x224),
        Register('dr10', 8, 0x228),
        Register('fr11', 4, 0x228),
        Register('fr10', 4, 0x22c),
        Register('fv12', 16, 0x230),
        Register('dr12', 8, 0x230),
        Register('fr13', 4, 0x230),
        Register('fr12', 4, 0x234),
        Register('dr14', 8, 0x238),
        Register('fr15', 4, 0x238),
        Register('fr14', 4, 0x23c),
        Register('xd0', 8, 0x240),
        Register('xf0', 4, 0x240),
        Register('xf1', 4, 0x244),
        Register('xd2', 8, 0x248),
        Register('xf2', 4, 0x248),
        Register('xf3', 4, 0x24c),
        Register('xd4', 8, 0x250),
        Register('xf4', 4, 0x250),
        Register('xf5', 4, 0x254),
        Register('xd6', 8, 0x258),
        Register('xf6', 4, 0x258),
        Register('xf7', 4, 0x25c),
        Register('xd8', 8, 0x260),
        Register('xf8', 4, 0x260),
        Register('xf9', 4, 0x264),
        Register('xd10', 8, 0x268),
        Register('xf10', 4, 0x268),
        Register('xf11', 4, 0x26c),
        Register('xd12', 8, 0x270),
        Register('xf12', 4, 0x270),
        Register('xf13', 4, 0x274),
        Register('xd14', 8, 0x278),
        Register('xf14', 4, 0x278),
        Register('xf15', 4, 0x27c),
        Register('gbr', 4, 0x400),
        Register('sr', 4, 0x404),
        Register('ssr', 4, 0x408),
        Register('spc', 4, 0x40c),
        Register('vbr', 4, 0x410),
        Register('sgr', 4, 0x414),
        Register('dbr', 4, 0x418),
        Register('md', 1, 0x600),
        Register('rb', 1, 0x601),
        Register('bl', 1, 0x602),
        Register('fd', 1, 0x603),
        Register('m', 1, 0x604),
        Register('q', 1, 0x605),
        Register('imask', 1, 0x606),
        Register('s', 1, 0x607),
        Register('t', 1, 0x608),
        Register('mach', 4, 0x800),
        Register('macl', 4, 0x804),
        Register('pr', 4, 0x808),
        Register('pc', 4, 0x80c, alias_names=('ip',)),
        Register('fpscr', 4, 0x810),
        Register('fpul', 4, 0x814),
        Register('fpscr_rm', 1, 0xa00),
        Register('fpscr_flag', 1, 0xa01),
        Register('fpscr_enable', 1, 0xa02),
        Register('fpscr_cause', 1, 0xa03),
        Register('fpscr_dn', 1, 0xa04),
        Register('fpscr_pr', 1, 0xa05),
        Register('fpscr_sz', 1, 0xa06),
        Register('fpscr_fr', 1, 0xa07)
    ]

register_arch(['superh4:le:32:default'], 32, Endness.LE, ArchPcode_SuperH4_LE_32_default)



###
### This file was automatically generated
###

from archinfo.arch import register_arch, Endness, Register

from .common import ArchPcode


class ArchPcode_Toy_LE_32_builder(ArchPcode):
    name = 'Toy:LE:32:builder'
    pcode_arch = 'Toy:LE:32:builder'
    description = 'Toy (test-builder) processor 32-bit little-endian'
    bits = 32
    ip_offset = 0x103c
    sp_offset = 0x1034
    bp_offset = sp_offset
    instruction_endness = Endness.LE
    register_list = [
        Register('r0', 4, 0x1000),
        Register('r0l', 2, 0x1000),
        Register('r0h', 2, 0x1002),
        Register('r1', 4, 0x1004),
        Register('r1l', 2, 0x1004),
        Register('r1h', 2, 0x1006),
        Register('r2', 4, 0x1008),
        Register('r2l', 2, 0x1008),
        Register('r2h', 2, 0x100a),
        Register('r3', 4, 0x100c),
        Register('r3l', 2, 0x100c),
        Register('r3h', 2, 0x100e),
        Register('r4', 4, 0x1010),
        Register('r4l', 2, 0x1010),
        Register('r4h', 2, 0x1012),
        Register('r5', 4, 0x1014),
        Register('r5l', 2, 0x1014),
        Register('r5h', 2, 0x1016),
        Register('r6', 4, 0x1018),
        Register('r6l', 2, 0x1018),
        Register('r6h', 2, 0x101a),
        Register('r7', 4, 0x101c),
        Register('r7l', 2, 0x101c),
        Register('r7h', 2, 0x101e),
        Register('r8', 4, 0x1020),
        Register('r8l', 2, 0x1020),
        Register('r8h', 2, 0x1022),
        Register('r9', 4, 0x1024),
        Register('r9l', 2, 0x1024),
        Register('r9h', 2, 0x1026),
        Register('r10', 4, 0x1028),
        Register('r10l', 2, 0x1028),
        Register('r10h', 2, 0x102a),
        Register('r11', 4, 0x102c),
        Register('r11l', 2, 0x102c),
        Register('r11h', 2, 0x102e),
        Register('r12', 4, 0x1030),
        Register('r12l', 2, 0x1030),
        Register('r12h', 2, 0x1032),
        Register('sp', 4, 0x1034),
        Register('spl', 2, 0x1034),
        Register('sph', 2, 0x1036),
        Register('lr', 4, 0x1038),
        Register('lrl', 2, 0x1038),
        Register('lrh', 2, 0x103a),
        Register('pc', 4, 0x103c, alias_names=('ip',)),
        Register('pcl', 2, 0x103c),
        Register('pch', 2, 0x103e),
        Register('c', 1, 0x1100),
        Register('z', 1, 0x1101),
        Register('n', 1, 0x1102),
        Register('v', 1, 0x1103),
        Register('contextreg', 8, 0x2000)
    ]

register_arch(['toy:le:32:builder'], 32, Endness.LE, ArchPcode_Toy_LE_32_builder)
